@@ -1,6 +1,6 @@
 #####################################################################
 ## AUTHOR: Mary Ehlers, regina.verbae@gmail.com
-## ABSTRACT: An initialized pipeline segment for Piper
+## ABSTRACT: An initialized pipeline segment for the Piper system
 #####################################################################
 
 package Piper::Instance;
@@ -9,7 +9,7 @@ use v5.10;
 use strict;
 use warnings;
 
-use List::AllUtils qw(last_value max part sum);
+use List::AllUtils qw(any last_value max part sum);
 use List::UtilsBy qw(max_by min_by);
 use Piper::Path;
 use Scalar::Util qw(weaken);
@@ -25,34 +25,30 @@ use overload (
     fallback => 1,
 );
 
-our $VERSION = '0.01'; # from Piper-0.01.tar.gz
+our $VERSION = '0.02'; # from Piper-0.02.tar.gz
 
-=head1 DESCRIPTION
-
-The object representing an initialized pipeline segment in the L<Piper> system.
-
-=head1 ATTRIBUTES
-
-=head2 batch_size
-
-The number of items to process at a time for this segment.
-
-If not set, inherits the C<batch_size> of any existing parent(s).  If the segment has no parents, or if none of its parents have a C<batch_size> defined, the default C<batch_size> will be used.  The default is 200, but this can be configured at import of L<Piper>.
-
-To clear a previously-set C<batch_size>, simply set it to C<undef> or use the C<clear_batch_size> method.
-
-    $segment->batch_size(undef);
-    $segment->clear_batch_size;
-
-=cut
+#pod =head1 ATTRIBUTES
+#pod
+#pod =head2 batch_size
+#pod
+#pod The number of items to process at a time for this segment.
+#pod
+#pod If not set, inherits the C<batch_size> of any existing parent(s).  If the segment has no parents, or if none of its parents have a C<batch_size> defined, the default C<batch_size> will be used.  The default is 200, but this can be configured at import of L<Piper>.
+#pod
+#pod To clear a previously-set C<batch_size>, simply set it to C<undef> or use the C<clear_batch_size> method.
+#pod
+#pod     $segment->batch_size(undef);
+#pod     $segment->clear_batch_size;
+#pod
+#pod =cut
 
 # around below to set up inheritance through parents
 
-=head2 children
-
-For container instances (made from L<Piper> objects, not L<Piper::Process> objects), the C<children> attribute holds an arrayref of the contained instance objects.
-
-=cut
+#pod =head2 children
+#pod
+#pod For container instances (made from L<Piper> objects, not L<Piper::Process> objects), the C<children> attribute holds an arrayref of the contained instance objects.
+#pod
+#pod =cut
 
 has children => (
     is => 'ro',
@@ -64,37 +60,37 @@ has children => (
     predicate => 1,
 );
 
-=head2 debug
-
-Debug level for this segment.  When accessing, inherits the debug level of any existing parent(s) if not explicitly set for this segment.  The default level is 0, but can be globally overridden with the environment variable C<PIPER_DEBUG>.
-
-To clear a previously-set debug level for a segment, simply set it to C<undef> or use the C<clear_debug> method.
-
-    $segment->debug(undef);
-    $segment->clear_debug;
-
-=cut
+#pod =head2 debug
+#pod
+#pod Debug level for this segment.  When accessing, inherits the debug level of any existing parent(s) if not explicitly set for this segment.  The default level is 0, but can be globally overridden with the environment variable C<PIPER_DEBUG>.
+#pod
+#pod To clear a previously-set debug level for a segment, simply set it to C<undef> or use the C<clear_debug> method.
+#pod
+#pod     $segment->debug(undef);
+#pod     $segment->clear_debug;
+#pod
+#pod =cut
 
 # around below to set up inheritance through parents
 
-=head2 enabled
-
-A boolean indicating that the segment is enabled and can accept items for processing.  Inherits this attribute from any existing parent(s) with a default of true.
-
-To clear a previously-set enabled attribute, simply set it to C<undef> or use the C<clear_enabled> method.
-
-    $segment->enabled(undef);
-    $segment->clear_enabled;
-
-=cut
+#pod =head2 enabled
+#pod
+#pod A boolean indicating that the segment is enabled and can accept items for processing.  Inherits this attribute from any existing parent(s) with a default of true.
+#pod
+#pod To clear a previously-set enabled attribute, simply set it to C<undef> or use the C<clear_enabled> method.
+#pod
+#pod     $segment->enabled(undef);
+#pod     $segment->clear_enabled;
+#pod
+#pod =cut
 
 # around below to set up inheritance through parents
 
-=head2 main
-
-Holds a reference to the outermost container instance for the pipeline.
-
-=cut
+#pod =head2 main
+#pod
+#pod Holds a reference to the outermost container instance for the pipeline.
+#pod
+#pod =cut
 
 has main => (
     is => 'lazy',
@@ -110,11 +106,11 @@ has main => (
     },
 );
 
-=head2 parent
-
-Unless this segment is the outermost container (C<main>), this attribute holds a reference to the segment's immediate container.
-
-=cut
+#pod =head2 parent
+#pod
+#pod Unless this segment is the outermost container (C<main>), this attribute holds a reference to the segment's immediate container.
+#pod
+#pod =cut
 
 has parent => (
     is => 'rwp',
@@ -125,11 +121,11 @@ has parent => (
     predicate => 1,
 );
 
-=head2 path
-
-The full path to this segment, built as the concatenation of all the parent(s) labels and the segment's label, joined by C</>.  L<Piper::Instance> objects stringify to this attribute.
-
-=cut
+#pod =head2 path
+#pod
+#pod The full path to this segment, built as the concatenation of all the parent(s) labels and the segment's label, joined by C</>.  L<Piper::Instance> objects stringify to this attribute.
+#pod
+#pod =cut
 
 has path => (
     is => 'lazy',
@@ -143,16 +139,16 @@ has path => (
     },
 );
 
-=head2 verbose
-
-Verbosity level for this segment.  When accessing, inherits verbosity level of any existing parent(s) if not explicitly set for this segment.
-
-To clear a previously-set verbosity level for a segment, simply set it to C<undef> or use the C<clear_verbose> method.
-
-    $segment->verbose(undef);
-    $segment->clear_verbose;
-
-=cut
+#pod =head2 verbose
+#pod
+#pod Verbosity level for this segment.  When accessing, inherits verbosity level of any existing parent(s) if not explicitly set for this segment.
+#pod
+#pod To clear a previously-set verbosity level for a segment, simply set it to C<undef> or use the C<clear_verbose> method.
+#pod
+#pod     $segment->verbose(undef);
+#pod     $segment->clear_verbose;
+#pod
+#pod =cut
 
 # Inherit parent settings
 for my $attr (qw(batch_size debug enabled verbose)) {
@@ -183,44 +179,58 @@ for my $attr (qw(batch_size debug enabled verbose)) {
     };
 }
 
-=head1 METHODS
+#pod =head1 METHODS
+#pod
+#pod Methods marked with a (*) should only be called from the outermost instance.
+#pod
+#pod =head2 clear_batch_size
+#pod
+#pod =head2 clear_debug
+#pod
+#pod =head2 clear_enabled
+#pod
+#pod =head2 clear_verbose
+#pod
+#pod Methods for clearing the corresponding attribute.
+#pod
+#pod =head2 has_children
+#pod
+#pod A boolean indicating whether the instance has any children (contained instances).  Will be true for all segments initialized from a L<Piper> object and false for all segments initialized from a L<Piper::Process> object.
+#pod
+#pod =head2 has_parent
+#pod
+#pod A boolean indicating whether the instance has a parent (container instance).  Will be true for all segments except the outermost segment (C<main>).
+#pod
+#pod =head2 has_pending
+#pod
+#pod Returns a boolean indicating whether there are any items that are queued at some level of the segment but have not completed processing.
+#pod
+#pod =cut
 
-Methods marked with a (*) should only be called from the outermost instance.
+sub has_pending {
+    my ($self) = @_;
 
-=head2 clear_batch_size
+    return $self->has_children
+        ? any { $_->has_pending } @{$self->children}
+        : $self->queue->ready;
+}
 
-=head2 clear_debug
-
-=head2 clear_enabled
-
-=head2 clear_verbose
-
-Methods for clearing the corresponding attribute.
-
-=head2 has_children
-
-A boolean indicating whether the instance has any children (contained instances).  Will be true for all segments initialized from a L<Piper> object and false for all segments initialized from a L<Piper::Process> object.
-
-=head2 has_parent
-
-A boolean indicating whether the instance has a parent (container instance).  Will be true for all segments except the outermost segment (C<main>).
-
-=head2 *dequeue([$num])
-
-Remove at most C<$num> S<(default 1)> processed items from the end of the segment.
-
-=head2 *enqueue(@data)
-
-Queue C<@data> for processing by the pipeline.
-
-=cut
+#pod =head2 *dequeue([$num])
+#pod
+#pod Remove at most C<$num> S<(default 1)> processed items from the end of the segment.
+#pod
+#pod =head2 *enqueue(@data)
+#pod
+#pod Queue C<@data> for processing by the pipeline.
+#pod
+#pod =cut
 
 around enqueue => sub {
     my ($orig, $self, @args) = @_;
 
     if (!$self->enabled) {
         # Bypass - go straight to drain
-        $self->INFO("Skipping disabled process", @args);
+        $self->INFO('Skipping disabled process', @args);
         $self->drain->enqueue(@args);
         return;
     }
@@ -234,7 +244,7 @@ around enqueue => sub {
         @items = @$queue if defined $queue;
 
         if (defined $skip) {
-            $self->INFO("Disallowed items emitted to next handler", @$skip);
+            $self->INFO('Disallowed items emitted to next handler', @$skip);
             $self->drain->enqueue(@$skip);
         }
     }
@@ -244,39 +254,39 @@ around enqueue => sub {
 
     return unless @items;
 
-    $self->INFO("Queueing items", @items);
+    $self->INFO('Queueing items', @items);
     $self->$orig(@items);
 };
 
-=head2 find_segment($location)
-
-Find and return the segment instance according to <$location>, which can be a label or a path-like hierarchy of labels.
-
-For example, in the following pipeline, a few possible C<$location> values include C<a>, C<subpipe/b>, or C<main/subpipe/c>.
-
-    my $pipe = Piper->new(
-        { label => 'main' },
-        subpipe => Piper->new(
-            a => sub { ... },
-            b => sub { ... },
-            c => sub { ... },
-        ),
-    )->init;
-
-If a label is unique within the pipeline, no path is required.  For non-unique labels, searches are performed in a nearest-neighbor, depth-first manner.
-
-For example, in the following pipeline, searching for C<processA> from C<processB> would find C<main/pipeA/processA>, not C<main/processA>.  So to reach C<main/processA> from C<processB>, the appropriate search would be for C<main/processA>.
-
-    my $pipe = Piper->new(
-        { label => 'main' },
-        pipeA => Piper->new(
-            processA => sub { ... },
-            processB => sub { ... },
-        ),
-        processA => sub { ... },
-    );
-
-=cut
+#pod =head2 find_segment($location)
+#pod
+#pod Find and return the segment instance according to <$location>, which can be a label or a path-like hierarchy of labels.
+#pod
+#pod For example, in the following pipeline, a few possible C<$location> values include C<a>, C<subpipe/b>, or C<main/subpipe/c>.
+#pod
+#pod     my $pipe = Piper->new(
+#pod         { label => 'main' },
+#pod         subpipe => Piper->new(
+#pod             a => sub { ... },
+#pod             b => sub { ... },
+#pod             c => sub { ... },
+#pod         ),
+#pod     )->init;
+#pod
+#pod If a label is unique within the pipeline, no path is required.  For non-unique labels, searches are performed in a nearest-neighbor, depth-first manner.
+#pod
+#pod For example, in the following pipeline, searching for C<processA> from C<processB> would find C<main/pipeA/processA>, not C<main/processA>.  So to reach C<main/processA> from C<processB>, the appropriate search would be for C<main/processA>.
+#pod
+#pod     my $pipe = Piper->new(
+#pod         { label => 'main' },
+#pod         pipeA => Piper->new(
+#pod             processA => sub { ... },
+#pod             processB => sub { ... },
+#pod         ),
+#pod         processA => sub { ... },
+#pod     );
+#pod
+#pod =cut
 
 sub find_segment {
     my ($self, $location) = @_;
@@ -308,11 +318,25 @@ sub find_segment {
     return $cache->{$location};
 }
 
-=head2 *is_exhausted
+#pod =head2 *flush
+#pod
+#pod Process batches until there are no more items pending.
+#pod
+#pod =cut
 
-Returns a boolean indicating whether there are any items left to process or dequeue.
+sub flush {
+    my ($self) = @_;
 
-=cut
+    while ($self->has_pending) {
+        $self->process_batch;
+    }
+}
+
+#pod =head2 *is_exhausted
+#pod
+#pod Returns a boolean indicating whether there are any items left to process or dequeue.
+#pod
+#pod =cut
 
 sub is_exhausted {
     my ($self) = @_;
@@ -320,22 +344,22 @@ sub is_exhausted {
     return $self->prepare ? 0 : 1;
 }
 
-=head2 *isnt_exhausted
-
-Returns the opposite of C<is_exhausted>.
-
-=cut
+#pod =head2 *isnt_exhausted
+#pod
+#pod Returns the opposite of C<is_exhausted>.
+#pod
+#pod =cut
 
 sub isnt_exhausted {
     my ($self) = @_;
     return !$self->is_exhausted;
 }
 
-=head2 next_segment
-
-Returns the next adjacent segment from the calling segment.  Returns C<undef> for the outermost container.
-
-=cut
+#pod =head2 next_segment
+#pod
+#pod Returns the next adjacent segment from the calling segment.  Returns C<undef> for the outermost container.
+#pod
+#pod =cut
 
 sub next_segment {
     my ($self) = @_;
@@ -343,11 +367,11 @@ sub next_segment {
     return $self->parent->follower->{$self};
 }
 
-=head2 pending
-
-Returns the number of items that are queued at some level of the segment but have not completed processing.
-
-=cut
+#pod =head2 pending
+#pod
+#pod Returns the number of items that are queued at some level of the segment but have not completed processing.
+#pod
+#pod =cut
 
 sub pending {
     my ($self) = @_;
@@ -359,74 +383,87 @@ sub pending {
     }
 }
 
-=head2 *prepare([$num])
-
-Process batches while data is still C<pending> until at least C<$num> S<(default 1)> items are C<ready> for C<dequeue>.
-
-=cut
+#pod =head2 *prepare([$num])
+#pod
+#pod Process batches while data is still C<pending> until at least C<$num> S<(default 1)> items are C<ready> for C<dequeue>.
+#pod
+#pod =cut
 
 sub prepare {
     my ($self, $num) = @_;
     $num //= 1;
 
-    while ($self->pending and $self->ready < $num) {
+    while ($self->has_pending and $self->ready < $num) {
         $self->process_batch;
     }
     return $self->ready;
 }
 
-=head2 ready
+#pod =head2 ready
+#pod
+#pod Returns the number of items that have finished processing and are ready for C<dequeue> from the segment.
+#pod
+#pod =cut
 
-Returns the number of items that have finished processing and are ready for C<dequeue> from the segment.
-
-=cut
-
-=head1 FLOW CONTROL METHODS
-
-These methods are available for use within process handler subroutines (see L<Piper::Process>).
-
-=head2 eject(@data)
-
-Send C<@data> to the drain of the outermost segment, making the C<@data> immediately ready for C<dequeue>.
-
-=cut
+#pod =head1 FLOW CONTROL METHODS
+#pod
+#pod These methods are available for use within process handler subroutines (see L<Piper::Process>).
+#pod
+#pod =head2 eject(@data)
+#pod
+#pod If the segment has a parent, send C<@data> to the drain of its parent.  Otherwise, enqueues C<@data> to the segment's drain.
+#pod
+#pod =cut
 
 sub eject {
     my $self = shift;
-    $self->INFO("Ejecting to drain", @_);
-    $self->main->drain->enqueue(@_);
+    if ($self->has_parent) {
+        $self->INFO('Ejecting to drain of parent ('.$self->parent.')', @_);
+        $self->parent->drain->enqueue(@_);
+    }
+    else {
+        $self->INFO('Ejecting to drain', @_);
+        $self->drain->enqueue(@_);
+    }
 }
 
-=head2 emit(@data)
-
-Send C<@data> to the next segment in the pipeline.  If the segment is the last in the pipeline, emits to the drain, making the C<@data> ready for C<dequeue>.
-
-=cut
+#pod =head2 emit(@data)
+#pod
+#pod Send C<@data> to the next segment in the pipeline.  If the segment is the last in the pipeline, emits to the drain, making the C<@data> ready for C<dequeue>.
+#pod
+#pod =cut
 
 sub emit {
     my $self = shift;
-    $self->INFO("Emitting", @_);
+    $self->INFO('Emitting', @_);
     # Just collect in the drain
     $self->drain->enqueue(@_);
 }
 
-=head2 inject(@data)
-
-Send C<@data> to the queue of the outermost segment.
-
-=cut
+#pod =head2 inject(@data)
+#pod
+#pod If the segment has a parent, enqueues C<@data> to its parent.  Otherwise, enqueues <@data> to itself.
+#pod
+#pod =cut
 
 sub inject {
     my $self = shift;
-    $self->INFO('Injecting to '.$self->main, @_);
-    $self->main->enqueue(@_);
+
+    if ($self->has_parent) {
+        $self->INFO('Injecting to parent ('.$self->parent.')', @_);
+        $self->parent->enqueue(@_);
+    }
+    else {
+        $self->INFO('Injecting to self ('.$self.')', @_);
+        $self->enqueue(@_);
+    }
 }
 
-=head2 injectAfter($location, @data)
-
-Send C<@data> to the segment I<after> the specified C<$location>.  See L<C<find_segment>|/find_segment($location)> for a detailed description of C<$location>.
-
-=cut
+#pod =head2 injectAfter($location, @data)
+#pod
+#pod Send C<@data> to the segment I<after> the specified C<$location>.  See L<C<find_segment>|/find_segment($location)> for a detailed description of C<$location>.
+#pod
+#pod =cut
 
 sub injectAfter {
     my $self = shift;
@@ -438,11 +475,11 @@ sub injectAfter {
     $segment->drain->enqueue(@_);
 }
 
-=head2 injectAt($location, @data)
-
-Send C<@data> to the segment I<at> the specified C<$location>.  See L<C<find_segment>|/find_segment($location)> for a detailed description of C<$location>.
-
-=cut
+#pod =head2 injectAt($location, @data)
+#pod
+#pod Send C<@data> to the segment I<at> the specified C<$location>.  See L<C<find_segment>|/find_segment($location)> for a detailed description of C<$location>.
+#pod
+#pod =cut
 
 sub injectAt {
     my $self = shift;
@@ -454,47 +491,47 @@ sub injectAt {
     $segment->enqueue(@_);
 }
 
-=head2 recycle(@data)
-
-Re-queue C<@data> to the top of the current segment in an order such that C<dequeue(1)> would subsequently return C<$data[0]> and so forth.
-
-=cut
+#pod =head2 recycle(@data)
+#pod
+#pod Re-queue C<@data> to the top of the current segment in an order such that C<dequeue(1)> would subsequently return C<$data[0]> and so forth.
+#pod
+#pod =cut
 
 sub recycle {
     my $self = shift;
-    $self->INFO("Recycling", @_);
+    $self->INFO('Recycling', @_);
     $self->requeue(@_);
 }
 
-=head1 LOGGING AND DEBUGGING METHODS
-
-See L<Piper::Logger> for detailed descriptions.
-
-=head2 INFO($message, [@items])
-
-Prints an informational C<$message> to STDERR if either the debug or verbosity level for the segment S<< is > 0 >>.
-
-=head2 DEBUG($message, [@items])
-
-Prints a debug C<$message> to STDERR if the debug level for the segment S<< is > 0 >>.
-
-=head2 WARN($message, [@items])
-
-Issues a warning with C<$message> via L<Carp::carp|Carp>.
-
-=head2 ERROR($message, [@items])
-
-Throws an error with C<$message> via L<Carp::croak|Carp>.
-
-=head1 UTILITY ATTRIBUTES
-
-None of these should be directly accessed.  Documented for contributors and source-code readers.
-
-=head2 args
-
-The arguments passed to the C<init> method of L<Piper>.
-
-=cut
+#pod =head1 LOGGING AND DEBUGGING METHODS
+#pod
+#pod See L<Piper::Logger> for detailed descriptions.
+#pod
+#pod =head2 INFO($message, [@items])
+#pod
+#pod Prints an informational C<$message> to STDERR if either the debug or verbosity level for the segment S<< is > 0 >>.
+#pod
+#pod =head2 DEBUG($message, [@items])
+#pod
+#pod Prints a debug C<$message> to STDERR if the debug level for the segment S<< is > 0 >>.
+#pod
+#pod =head2 WARN($message, [@items])
+#pod
+#pod Issues a warning with C<$message> via L<Carp::carp|Carp>.
+#pod
+#pod =head2 ERROR($message, [@items])
+#pod
+#pod Throws an error with C<$message> via L<Carp::croak|Carp>.
+#pod
+#pod =head1 UTILITY ATTRIBUTES
+#pod
+#pod None of these should be directly accessed.  Documented for contributors and source-code readers.
+#pod
+#pod =head2 args
+#pod
+#pod The arguments passed to the C<init> method of L<Piper>.
+#pod
+#pod =cut
 
 has args => (
     is => 'rwp',
@@ -511,11 +548,11 @@ has args => (
     },
 );
 
-=head2 directory
-
-A hashref of the segment's children, keyed by their labels.  Used by C<find_segment>.
-
-=cut
+#pod =head2 directory
+#pod
+#pod A hashref of the segment's children, keyed by their labels.  Used by C<find_segment>.
+#pod
+#pod =cut
 
 has directory => (
     is => 'lazy',
@@ -531,11 +568,11 @@ has directory => (
     },
 );
 
-=head2 drain
-
-A reference to the location where the segment's processed items are emitted.
-
-=cut
+#pod =head2 drain
+#pod
+#pod A reference to the location where the segment's processed items are emitted.
+#pod
+#pod =cut
 
 BEGIN { # Enables 'with Piper::Role::Queue'
 has drain => (
@@ -553,11 +590,11 @@ has drain => (
 );
 }
 
-=head2 follower
-
-A hashref of children paths to the child's next adjacent segment.  Used by C<next_segment>.
-
-=cut
+#pod =head2 follower
+#pod
+#pod A hashref of children paths to the child's next adjacent segment.  Used by C<next_segment>.
+#pod
+#pod =cut
 
 has follower => (
     is => 'lazy',
@@ -579,11 +616,11 @@ has follower => (
     },
 );
 
-=head2 logger
-
-A reference to the logger for the pipeline.  Handles L</LOGGING AND DEBUGGING> methods.
-
-=cut
+#pod =head2 logger
+#pod
+#pod A reference to the logger for the pipeline.  Handles L</LOGGING AND DEBUGGING> methods.
+#pod
+#pod =cut
 
 has logger => (
     is => 'lazy',
@@ -614,11 +651,11 @@ around [qw(INFO DEBUG WARN ERROR)] => sub {
     }
 };
 
-=head2 queue
-
-A reference to the location where data is queued for processing by this segment.
-
-=cut
+#pod =head2 queue
+#pod
+#pod A reference to the location where data is queued for processing by this segment.
+#pod
+#pod =cut
 
 BEGIN { # Enables 'with Piper::Role::Queue'
 has queue => (
@@ -637,11 +674,11 @@ has queue => (
 );
 }
 
-=head2 segment
-
-The L<Piper> or L<Piper::Process> object from which the instance segment was created.
-
-=cut
+#pod =head2 segment
+#pod
+#pod The L<Piper> or L<Piper::Process> object from which the instance segment was created.
+#pod
+#pod =cut
 
 BEGIN { # So we can 'around' on Piper::Role::Segment methods
 has segment => (
@@ -652,15 +689,15 @@ has segment => (
 );
 }
 
-=head1 UTILITY METHODS
-
-None of these should be directly accessed.  Documented for contributors and source-code readers.
-
-=head2 descendant($path, $referrer)
-
-Returns a child segment if its path ends with C<$path>.  Does not search children with a path of C<$referrer>, as it was presumably already searched by a previous iteration of the search.  Used by C<find_segment>.
-
-=cut
+#pod =head1 UTILITY METHODS
+#pod
+#pod None of these should be directly accessed.  Documented for contributors and source-code readers.
+#pod
+#pod =head2 descendant($path, $referrer)
+#pod
+#pod Returns a child segment if its path ends with C<$path>.  Does not search children with a path of C<$referrer>, as it was presumably already searched by a previous iteration of the search.  Used by C<find_segment>.
+#pod
+#pod =cut
 
 sub descendant {
     my ($self, $path, $referrer) = @_;
@@ -668,7 +705,7 @@ sub descendant {
     $referrer //= '';
 
     $self->DEBUG("Searching for location '$path'");
-    $self->DEBUG("Referrer", $referrer) if $referrer;
+    $self->DEBUG('Referrer', $referrer) if $referrer;
 
     # Search immediate children
     $path = Piper::Path->new($path) if $path and not ref $path;
@@ -713,7 +750,7 @@ sub descendant {
         my $overlap = $self->label;
         if ($path =~ m{^\Q$overlap\E(?:$|/(?<path>.*))}) {
             $path = $+{path} // '';
-            $self->DEBUG("Overlapping descendant search", $path ? $path : ());
+            $self->DEBUG('Overlapping descendant search', $path ? $path : ());
             $descend = $path ? $self->descendant($path, $self) : $self;
         }
     }
@@ -721,11 +758,11 @@ sub descendant {
     return $descend;
 }
 
-=head2 pressure
-
-An integer metric for the "fullness" of the pending queue.  For handler instances (initialized from L<Piper::Process> objects), it is the percentage of pending items vs the batch size of the segment.  For container instances (initialized from L<Piper> objects), is is the maximum C<pressure> of the contained instances.  Used by process_batch for choosing which segment to process.
-
-=cut
+#pod =head2 pressure
+#pod
+#pod An integer metric for the "fullness" of the pending queue.  For handler instances (initialized from L<Piper::Process> objects), it is the percentage of pending items vs the batch size of the segment.  For container instances (initialized from L<Piper> objects), is is the maximum C<pressure> of the contained instances.  Used by process_batch for choosing which segment to process.
+#pod
+#pod =cut
 
 # Metric for "how full" the pending queue is
 sub pressure {
@@ -738,13 +775,13 @@ sub pressure {
     }
 }
 
-=head2 process_batch
-
-Chooses the "best" segment for processing, and processes a batch for that segment.
-
-It first attempts to choose the full-batch segment (C<< pending >= batch_size >>) closest to the end of the pipeline.  If there are no full-batch segments, it chooses the segment closest to being full.
-
-=cut
+#pod =head2 process_batch
+#pod
+#pod Chooses the "best" segment for processing, and processes a batch for that segment.
+#pod
+#pod It first attempts to choose the full-batch segment (C<< pending >= batch_size >>) closest to the end of the pipeline.  If there are no full-batch segments, it chooses the segment closest to being full.
+#pod
+#pod =cut
 
 sub process_batch {
     my ($self) = @_;
@@ -763,10 +800,10 @@ sub process_batch {
     }
     else {
         my $num = $self->batch_size;
-        $self->DEBUG("Processing batch with max size", $num);
+        $self->DEBUG('Processing batch with max size', $num);
 
         my @batch = $self->queue->dequeue($num);
-        $self->INFO("Processing batch", @batch);
+        $self->INFO('Processing batch', @batch);
 
         $self->segment->handler->(
             $self,
@@ -780,6 +817,258 @@ sub process_batch {
 
 __END__
 
+=pod
+
+=for :stopwords Mary Ehlers Heaney Tim
+
+=head1 NAME
+
+Piper::Instance - An initialized pipeline segment for the Piper system
+
+=head1 ATTRIBUTES
+
+=head2 batch_size
+
+The number of items to process at a time for this segment.
+
+If not set, inherits the C<batch_size> of any existing parent(s).  If the segment has no parents, or if none of its parents have a C<batch_size> defined, the default C<batch_size> will be used.  The default is 200, but this can be configured at import of L<Piper>.
+
+To clear a previously-set C<batch_size>, simply set it to C<undef> or use the C<clear_batch_size> method.
+
+    $segment->batch_size(undef);
+    $segment->clear_batch_size;
+
+=head2 children
+
+For container instances (made from L<Piper> objects, not L<Piper::Process> objects), the C<children> attribute holds an arrayref of the contained instance objects.
+
+=head2 debug
+
+Debug level for this segment.  When accessing, inherits the debug level of any existing parent(s) if not explicitly set for this segment.  The default level is 0, but can be globally overridden with the environment variable C<PIPER_DEBUG>.
+
+To clear a previously-set debug level for a segment, simply set it to C<undef> or use the C<clear_debug> method.
+
+    $segment->debug(undef);
+    $segment->clear_debug;
+
+=head2 enabled
+
+A boolean indicating that the segment is enabled and can accept items for processing.  Inherits this attribute from any existing parent(s) with a default of true.
+
+To clear a previously-set enabled attribute, simply set it to C<undef> or use the C<clear_enabled> method.
+
+    $segment->enabled(undef);
+    $segment->clear_enabled;
+
+=head2 main
+
+Holds a reference to the outermost container instance for the pipeline.
+
+=head2 parent
+
+Unless this segment is the outermost container (C<main>), this attribute holds a reference to the segment's immediate container.
+
+=head2 path
+
+The full path to this segment, built as the concatenation of all the parent(s) labels and the segment's label, joined by C</>.  L<Piper::Instance> objects stringify to this attribute.
+
+=head2 verbose
+
+Verbosity level for this segment.  When accessing, inherits verbosity level of any existing parent(s) if not explicitly set for this segment.
+
+To clear a previously-set verbosity level for a segment, simply set it to C<undef> or use the C<clear_verbose> method.
+
+    $segment->verbose(undef);
+    $segment->clear_verbose;
+
+=head1 METHODS
+
+Methods marked with a (*) should only be called from the outermost instance.
+
+=head2 clear_batch_size
+
+=head2 clear_debug
+
+=head2 clear_enabled
+
+=head2 clear_verbose
+
+Methods for clearing the corresponding attribute.
+
+=head2 has_children
+
+A boolean indicating whether the instance has any children (contained instances).  Will be true for all segments initialized from a L<Piper> object and false for all segments initialized from a L<Piper::Process> object.
+
+=head2 has_parent
+
+A boolean indicating whether the instance has a parent (container instance).  Will be true for all segments except the outermost segment (C<main>).
+
+=head2 has_pending
+
+Returns a boolean indicating whether there are any items that are queued at some level of the segment but have not completed processing.
+
+=head2 *dequeue([$num])
+
+Remove at most C<$num> S<(default 1)> processed items from the end of the segment.
+
+=head2 *enqueue(@data)
+
+Queue C<@data> for processing by the pipeline.
+
+=head2 find_segment($location)
+
+Find and return the segment instance according to <$location>, which can be a label or a path-like hierarchy of labels.
+
+For example, in the following pipeline, a few possible C<$location> values include C<a>, C<subpipe/b>, or C<main/subpipe/c>.
+
+    my $pipe = Piper->new(
+        { label => 'main' },
+        subpipe => Piper->new(
+            a => sub { ... },
+            b => sub { ... },
+            c => sub { ... },
+        ),
+    )->init;
+
+If a label is unique within the pipeline, no path is required.  For non-unique labels, searches are performed in a nearest-neighbor, depth-first manner.
+
+For example, in the following pipeline, searching for C<processA> from C<processB> would find C<main/pipeA/processA>, not C<main/processA>.  So to reach C<main/processA> from C<processB>, the appropriate search would be for C<main/processA>.
+
+    my $pipe = Piper->new(
+        { label => 'main' },
+        pipeA => Piper->new(
+            processA => sub { ... },
+            processB => sub { ... },
+        ),
+        processA => sub { ... },
+    );
+
+=head2 *flush
+
+Process batches until there are no more items pending.
+
+=head2 *is_exhausted
+
+Returns a boolean indicating whether there are any items left to process or dequeue.
+
+=head2 *isnt_exhausted
+
+Returns the opposite of C<is_exhausted>.
+
+=head2 next_segment
+
+Returns the next adjacent segment from the calling segment.  Returns C<undef> for the outermost container.
+
+=head2 pending
+
+Returns the number of items that are queued at some level of the segment but have not completed processing.
+
+=head2 *prepare([$num])
+
+Process batches while data is still C<pending> until at least C<$num> S<(default 1)> items are C<ready> for C<dequeue>.
+
+=head2 ready
+
+Returns the number of items that have finished processing and are ready for C<dequeue> from the segment.
+
+=head1 FLOW CONTROL METHODS
+
+These methods are available for use within process handler subroutines (see L<Piper::Process>).
+
+=head2 eject(@data)
+
+If the segment has a parent, send C<@data> to the drain of its parent.  Otherwise, enqueues C<@data> to the segment's drain.
+
+=head2 emit(@data)
+
+Send C<@data> to the next segment in the pipeline.  If the segment is the last in the pipeline, emits to the drain, making the C<@data> ready for C<dequeue>.
+
+=head2 inject(@data)
+
+If the segment has a parent, enqueues C<@data> to its parent.  Otherwise, enqueues <@data> to itself.
+
+=head2 injectAfter($location, @data)
+
+Send C<@data> to the segment I<after> the specified C<$location>.  See L<C<find_segment>|/find_segment($location)> for a detailed description of C<$location>.
+
+=head2 injectAt($location, @data)
+
+Send C<@data> to the segment I<at> the specified C<$location>.  See L<C<find_segment>|/find_segment($location)> for a detailed description of C<$location>.
+
+=head2 recycle(@data)
+
+Re-queue C<@data> to the top of the current segment in an order such that C<dequeue(1)> would subsequently return C<$data[0]> and so forth.
+
+=head1 LOGGING AND DEBUGGING METHODS
+
+See L<Piper::Logger> for detailed descriptions.
+
+=head2 INFO($message, [@items])
+
+Prints an informational C<$message> to STDERR if either the debug or verbosity level for the segment S<< is > 0 >>.
+
+=head2 DEBUG($message, [@items])
+
+Prints a debug C<$message> to STDERR if the debug level for the segment S<< is > 0 >>.
+
+=head2 WARN($message, [@items])
+
+Issues a warning with C<$message> via L<Carp::carp|Carp>.
+
+=head2 ERROR($message, [@items])
+
+Throws an error with C<$message> via L<Carp::croak|Carp>.
+
+=head1 UTILITY ATTRIBUTES
+
+None of these should be directly accessed.  Documented for contributors and source-code readers.
+
+=head2 args
+
+The arguments passed to the C<init> method of L<Piper>.
+
+=head2 directory
+
+A hashref of the segment's children, keyed by their labels.  Used by C<find_segment>.
+
+=head2 drain
+
+A reference to the location where the segment's processed items are emitted.
+
+=head2 follower
+
+A hashref of children paths to the child's next adjacent segment.  Used by C<next_segment>.
+
+=head2 logger
+
+A reference to the logger for the pipeline.  Handles L</LOGGING AND DEBUGGING> methods.
+
+=head2 queue
+
+A reference to the location where data is queued for processing by this segment.
+
+=head2 segment
+
+The L<Piper> or L<Piper::Process> object from which the instance segment was created.
+
+=head1 UTILITY METHODS
+
+None of these should be directly accessed.  Documented for contributors and source-code readers.
+
+=head2 descendant($path, $referrer)
+
+Returns a child segment if its path ends with C<$path>.  Does not search children with a path of C<$referrer>, as it was presumably already searched by a previous iteration of the search.  Used by C<find_segment>.
+
+=head2 pressure
+
+An integer metric for the "fullness" of the pending queue.  For handler instances (initialized from L<Piper::Process> objects), it is the percentage of pending items vs the batch size of the segment.  For container instances (initialized from L<Piper> objects), is is the maximum C<pressure> of the contained instances.  Used by process_batch for choosing which segment to process.
+
+=head2 process_batch
+
+Chooses the "best" segment for processing, and processes a batch for that segment.
+
+It first attempts to choose the full-batch segment (C<< pending >= batch_size >>) closest to the end of the pipeline.  If there are no full-batch segments, it chooses the segment closest to being full.
+
 =head1 SEE ALSO
 
 =over
@@ -792,4 +1081,12 @@ __END__
 
 =back
 
+=head1 VERSION
+
+version 0.02
+
+=head1 AUTHOR
+
+Mary Ehlers <ehlers@cpan.org>
+
 =cut
